/**
 * Name: ct2c
 * Description: t2c generated C language.
 * Author: SSDK <ssdk@ispras.ru>
 * based on C++ programmer language by
 * Markku Rossi <mtr@iki.fi>
 */
 
sub macro_face (on)
{
  if (on)
    face_on (face_macro);
  else
    face_off (face_macro);
}

sub const_face (on)
{
  if (on)
    face_on (face_const);
  else
    face_off (face_const);
}

ct2c_macro_re =
/* Macros used in ct2c.
   (build-re   (ABORT_TEST_PURPOSE
		ABORT_UNSUPPORTED
                BEGIN_FINALLY_SECTION
	        END_FINALLY_SECTION
	        INIT_FAILED
	        REQ
	        RETURN
	        SET_STARTUP_FAILED
	        STARTUP_FAILED
	        T2C_GET_DATA_PATH
	        T2C_GET_STATUS
	        T2C_GET_TEST_PATH
	        T2C_PUT_STATUS
	        T2C_RESET_STATUS
	        TODO_REQ
	        TRACE
	        TRACE_NO_JOURNAL
	        TRACE0
	        UNRES
	        UNRES_ERR
	        UNRES_IF_STARTUP_FAILED))
*/
  /\b(ABORT_(TEST_PURPOSE|UNSUPPORTED)|BEGIN_FINALLY_SECTION|END_FINALLY_SECTION|INIT_FAILED|RE(Q|TURN)\
|S(ET_STARTUP_FAILED|TARTUP_FAILED)|T(2C_(GET_(DATA_PATH|STATUS|TEST_PATH)|PUT_STATUS\
|RESET_STATUS)|ODO_REQ|RAC(E|E_NO_JOURNAL|E0))|UNRE(S|S_ERR|S_IF_STARTUP_FAILED))\b/;

ct2c_const_re =
/* Constants used in ct2c
   (build-re   (FALSE
		NULL
		TRUE))
*/
  /\b(FALSE|NULL|TRUE)\b/;

/*
 * We inherit the C++ state from the C state.  This gives us all the
 * defaults, etc.  All we have to do here is to overwrite things that
 * are not implemented, or are broken.
 */
state ct2c extends c
{
  /*
   * Macros used in ct2c.
   */
  ct2c_macro_re {
    macro_face (true);
    language_print ($0);
    macro_face (false);
  }
  
  /*
   * Constants used in ct2c.
   */
  ct2c_const_re {
    const_face (true);
    language_print ($0);
    const_face (false);
  }
  
 /* Remove false labels. */
  /[a-zA-Z0-9_]+::/ {
    /*language_print ($2);*/
  }

  /* Labels.  Emacs accepts also bare numbers. */
  /^([ \t]*)([a-zA-Z0-9_]+)(:)/ {
    language_print ($1);

    if (strcmp ($2, "public") == 0
	|| strcmp ($2, "private") == 0
	|| strcmp ($2, "protected") == 0)
      {
	/* These use the `type face. */
	type_face (true);
	language_print ($2);
	type_face (false);
      }
    else
      {
	reference_face (true);
	language_print ($2);
	reference_face (false);
      }

    language_print ($3);
  }

  /*
   * Function definitions, but only if you code with the one and only
   * usable indentation style (GNU).
   */
  /^([a-zA-Z_][a-zA-Z_0-9:~]*)([ \t]*\()/ {
    function_name_face (true);
    language_print ($1);
    function_name_face (false);

    language_print ($2);
  }

  /* Function definitions and prototypes for other (loser) coding styles. */
  /^([A-Za-z][a-zA-Z0-9_\&\* ]+)([ \*])([a-zA-Z_][a-zA-Z_0-9:~]*)([ \t]*\()/ {
    garbage = $1;
    middle_garbage = $2;
    function_name = $3;
    tail_garbage = $4;

    highlight_types (garbage, type_re);

    language_print (middle_garbage);

    function_name_face (true);
/*    if( regmatch(function_name,/_spec$/i) )
	print("<a name='",function_name,"'></a>");
*/
    language_print (function_name);
    function_name_face (false);

    language_print (tail_garbage);
  }
}


/*
Local variables:
mode: c
End:
*/
