#!/bin/sh

TESTS="tests"
INCLUDE="include"
ANCHORER="anchorer.plx"
ARGS_MUST_BE=2
E_BADARGS=65
E_CANT_MKDIR=60

#
# Usage
#
function usage ()
{
    echo "This script generates the colored .html representation of test cases"
    echo "Usage: html_generator [Dir 1] [Dir 2]"
    echo "	 Where  Dir 1 is the directory of the test suites"
    echo "	 and    Dir 2 is the directory of the created html files"
    exit ${E_BADARGS}
}

#
# The recursive function for the directory/file tree
#
function recurse ()
{
    for HTML in *.html
    do
	rm -f ${HTML}
    done
    
    for DIR in *
    do
	if [ -d ${DIR} ]
	then
	    cd $2
    	    if [ ! -e ${DIR} ] 
	    then
		mkdir ${DIR}
	    else
		if [ ! -d ${DIR} ]
		then
		    echo "Can't make the ${DIR} directory"
		    exit ${E_CANT_MKDIR}
		fi
	    fi	
	    	    
	    cd $1/${DIR}
	    recurse $1/${DIR} $2/${DIR}
	    cd ..
	else
	    if [ ${DIR} != "*" ]
    	    then
		enscript -p- --language=ct2chtml --pretty-print=ct2c --style=ct2c --color ${DIR} > $2/${DIR}.html 2> /dev/null			
		if [ $? -ne 0 ]
		then
		    echo "Cant convert the file: ${DIR} into .html format"
		fi
		cd $2
		perl ${HOME_DIR}/${ANCHORER} ${DIR}.html 2> /dev/null
		if [ $? -ne 0 ]
		then
		    echo "Cant make anchors for the file: ${DIR}.html"
		fi
		rename .html.tmp .html ${DIR}.html.tmp
		cd $1
	    fi
	fi
    done
}

#
# Checking the parameters
#
if [ $# -ne ${ARGS_MUST_BE} ]
then
    echo "There must be ${ARGS_MUST_BE} parameters"
    usage
fi

if [ ! -d $1 ]
then
    echo "The first parameter is not the directory"
    usage
fi

if [ ! -r $1 ]
then
    echo "Have no rights to read the directory: $1"
    usage
fi

if [ ! -e $2 ] 
then
    mkdir -p $2
else
    if [ ! -d $2 ]
    then
	echo "The second parameter is a file"
        usage
    fi

    if [ ! -w $2 ]
    then
	echo "Have no rights to write in the directory $2"
	usage
    fi
fi	

#
# Geting the full path of directories 
# in DIR1, DIR2 and HOME_DIR
#

if [ ${1:0:1} == / ]
then
    DIR1=$1
else
    DIR1=${PWD}/$1
fi

if [ ${2:0:1} == / ]
then
    DIR2=$2
else
    DIR2=${PWD}/$2
fi

if [ ${0:0:1} == / ]
then
    HOME_DIR=`dirname $0`
else
    HOME_DIR=${PWD}/`dirname $0`
fi

#
# Test suites
#
cd ${DIR1}
for T2C1 in *-t2c
do
    if [ -d ${T2C1} ]
    then
	echo "Creating .html files for ${T2C1}..."
	
	#
	# Creating the test suite directory
	#
	T2C2=${T2C1}
	T2C1=${DIR1}/${T2C1}
	cd ${DIR2}
	if [ ! -e ${T2C2} ] 
	then
	    mkdir ${T2C2}
	else
	    if [ ! -d ${T2C2} ]
	    then
		echo "Can't make the ${T2C2} directory"
		exit ${E_CANT_MKDIR}
	    fi
	fi	
	T2C2=${DIR2}/${T2C2}
	
	#
	# Tests
	#	
	cd ${T2C1}
	if [ -d ${TESTS} ]
	then	
	    #
	    # Creating the tests directory
	    #
	    cd ${T2C2}
    	    if [ ! -e ${TESTS} ] 
	    then
		mkdir ${TESTS}
	    else
		if [ ! -d ${TESTS} ]
		then
		    echo "Can't make the ${TESTS} directory"
		    exit ${E_CANT_MKDIR}
		fi
	    fi	
	    TESTS1=${T2C1}/${TESTS}
	    TESTS2=${T2C2}/${TESTS}
	    
	    #
	    # Getting the FOLDERs
	    #
	    cd ${TESTS1}
	    for FOLDER1 in *
	    do
		if [ -d ${FOLDER1} ]
		then
		    
    		    #
		    # Creating the FOLDER directory
		    #
		    cd ${TESTS2}
		    FOLDER2=${FOLDER1}
    		    if [ ! -e ${FOLDER2} ] 
		    then
			mkdir ${FOLDER2}
		    else
			if [ ! -d ${FOLDER2} ]
			then
			    echo "Can't make the ${FOLDER2} directory"
			    exit ${E_CANT_MKDIR}
			fi
		    fi
		    FOLDER1=${TESTS1}/${FOLDER2}
		    FOLDER2=${TESTS2}/${FOLDER2}
		    
		    #
		    # Convertint the .c file
		    #
		    cd ${FOLDER1}
		    for C_FILE in *.c
		    do
			rm -f ${FOLDER2}/${C_FILE%.c}.html
			enscript -p- --language=ct2chtml --pretty-print=ct2c --style=ct2c --color ${C_FILE} > ${FOLDER2}/${C_FILE%.c}.html 2> /dev/null
			if [ $? -ne 0 ]
			then
			    echo "Cant convert the file: ${C_FILE} into .html format"
			fi
			cd ${FOLDER2}
			perl ${HOME_DIR}/${ANCHORER} ${C_FILE%.c}.html 2> /dev/null
			if [ $? -ne 0 ]
			then
			    echo "Cant make anchors for the file: ${C_FILE%.c}.html"
			fi
			rename .html.tmp .html ${C_FILE%.c}.html.tmp
			cd ${FOLDER1}
		    done
		    cd ${TESTS1}
		fi	    
	    done	    	
	fi
		
	#
	# Include
	#
	cd ${T2C1}
	if [ -d ${INCLUDE} ]
	then	
	    #
	    # Creating the include directory
	    #
	    cd ${T2C2}
    	    if [ ! -e ${INCLUDE} ] 
	    then
		mkdir ${INCLUDE}
	    else
		if [ ! -d ${INCLUDE} ]
		then
		    echo "Can't make the ${INCLUDE} directory"
		    exit ${E_CANT_MKDIR}
		fi
	    fi	
	    INCLUDE1=${T2C1}/${INCLUDE}
	    INCLUDE2=${T2C2}/${INCLUDE}
	    
	    #
	    # converting the files from the include directory
	    #	
	    cd ${INCLUDE1}
	    recurse ${INCLUDE1} ${INCLUDE2}
	fi
    fi
    cd ${DIR1}
done






