#!/bin/sh

TAB=";		"
COLUMN_LIBRARY="Library"
COLUMN_INTERFACE="Interface"
COLUMN_TC="Test Case"
COLUMN_SOURCE="Test Source"
COLUMN_STATUS="Status"
TESTS_PATTERN="tests"

TABLE_GENERATOR="table_generator.plx"
TABLE_GENERATOR_PATCH="tg_patch.plx";
ARGS_MUST_BE=2
E_BADARGS=65

Pwd=${PWD}

#
# Usage
#
function usage ()
{
    echo "This script generates the test case table"
    echo "Usage: tc_table_generator [Dir] [File] [Patch1...]"
    echo "	 Where  Dir is the directory of the test suites"
    echo "	 	File is the table file"
    echo "		Patch1, Patch2... are the patch files"
    exit ${E_BADARGS}
}

#
# Checking the parameters
#
if [ $# -lt ${ARGS_MUST_BE} ]
then
    echo "There must be at least ${ARGS_MUST_BE} parameters"
    usage
fi

if [ ! -d $1 ]
then
    echo "The first parameter is not the directory"
    usage
fi

if [ ! -r $1 ]
then
    echo "Have no rights to read the directory: $1"
    usage
fi

if [ -e $2 ] 
then
    if [ -d $2 ]
    then
	echo "The second parameter is the directory"
        usage
    fi

    if [ ! -w $2 ]
    then
	echo "Have no rights to write in the file $2"
	usage
    fi
fi

#
# Geting the full path of directories 
# in DIR, FILE, T_GENERATOR and TG_PATCH
#
if [ ${1:0:1} == / ]
then
    DIR=$1
else
    DIR=${PWD}/$1
fi

if [ ${2:0:1} == / ]
then
    FILE=$2
else
    FILE=${PWD}/$2
fi

if [ ${0:0:1} == / ]
then
    HOME_DIR=`dirname $0`
else
    HOME_DIR=${PWD}/`dirname $0`
fi
T_GENERATOR=${HOME_DIR}/${TABLE_GENERATOR}
TG_PATCH=${HOME_DIR}/${TABLE_GENERATOR_PATCH}

#
# Removing the FILE
#
#rm -f ${FILE}
echo "${COLUMN_LIBRARY}${TAB}${COLUMN_INTERFACE}${TAB}${COLUMN_TC}${TAB}${COLUMN_SOURCE}${TAB}${COLUMN_STATUS}" > ${FILE}

#
# Test suites
#
cd ${DIR}
for T2C in *-t2c
do
    if [ -d ${T2C} ]
    then
	echo "Creating the table file for ${T2C}..."
	
	#
	# Tests
	#	
	T2C_=/${T2C}
	T2C=${DIR}/${T2C}
	cd ${T2C}
	if [ -d ${TESTS_PATTERN} ]
	then
	    TESTS_=${T2C_}/${TESTS_PATTERN}
	    TESTS=${T2C}/${TESTS_PATTERN}
	    
	    #
	    # Getting the FOLDERs
	    #
	    cd ${TESTS}
	    for FOLDER in *
	    do
		if [ -d ${FOLDER} ]
		then
		    FOLDER_=${TESTS_}/${FOLDER}
		    FOLDER=${TESTS}/${FOLDER}
		    
		    #
		    # Generating the table file from the .c file
		    #
		    cd ${FOLDER}
		    for C_FILE in *.c
		    do
			PAR1=${FOLDER}/${C_FILE}
			PAR2=${FILE}
			PAR3=${FOLDER_}/${C_FILE%.c}
			perl ${T_GENERATOR} ${PAR1} ${PAR2} ${PAR3}			
			if [ $? -ne 0 ]
			then
			    echo "Cant make table file for the file: ${C_FILE}"
			fi
		    done
		    cd ${TESTS}
		fi	    
	    done	    	
	fi
    fi
    cd ${DIR}
done

arg_index=1
for arg in $@
do
    if [ $arg_index -lt 3 ]
    then
	let "arg_index+=1"
	continue
    fi
    
    if [ ${arg:0:1} == / ]
    then
	PATCH_FILE=$arg
    else
        PATCH_FILE=${Pwd}/$arg
    fi
    
    if [ ! -e $PATCH_FILE ] || [ -d $PATCH_FILE ]
    then
	echo "The file $arg doesn't exist (argument num:$arg_index)"
	let "arg_index+=1"
	continue
    fi
    
    perl $TG_PATCH $FILE $PATCH_FILE
    rename ".tmp" "" $FILE.tmp 
    
    let "arg_index+=1"
done












